
USE [master]
GO

SET NOCOUNT ON
GO

-- >> Server Uptime <<
select cast((datediff(hh, create_date, getdate()))/24 as varchar(3)) + ' days, '
     + cast((datediff(hh, create_date, getdate())) % 24 as varchar(2)) + ' hours'
     as [SQL Server Service Uptime]
from sys.databases where name = 'tempdb'
GO

----------------------------------------------------------------------------------------------------

-- >> Top 100 Expensive Queries <<
SELECT TOP 100 SUBSTRING (st.text, (qs.statement_start_offset / 2) + 1, ((CASE statement_end_offset
                                                                              WHEN-1 THEN DATALENGTH (st.text) 
                                                                              ELSE qs.statement_end_offset
                                                                          END - qs.statement_start_offset) / 2) + 1) AS statement_text, 
    CASE
	   WHEN st.text LIKE '%LIKE%' THEN 'LIKE'
	   ELSE ''
    END AS [Wildcard], 
    CASE
	   WHEN ph.query_plan.exist ('declare namespace ns="http://schemas.microsoft.com/sqlserver/2004/07/showplan";(/ns:ShowPlanXML/ns:BatchSequence/ns:Batch/ns:Statements/ns:StmtSimple/ns:QueryPlan/ns:RelOp/ns:OutputList/ns:ColumnReference/@Database)[1]') = 0 THEN ''
	   ELSE ph.query_plan.value ('declare namespace ns="http://schemas.microsoft.com/sqlserver/2004/07/showplan";(/ns:ShowPlanXML/ns:BatchSequence/ns:Batch/ns:Statements/ns:StmtSimple/ns:QueryPlan/ns:RelOp/ns:OutputList/ns:ColumnReference/@Database)[1]', 'nvarchar (max)') 
    END AS db,
    execution_count, 
    CASE
	   WHEN execution_count = 0 THEN NULL
	   ELSE total_logical_reads / execution_count
    END AS avg_logical_reads, 
    CASE
	   WHEN execution_count = 0 THEN NULL
	   ELSE (total_worker_time / execution_count) / 1000
    END AS [avg_cpu_(msec)], 
    CASE
	   WHEN execution_count = 0 THEN NULL
	   ELSE (total_elapsed_time / execution_count) / 1000
    END AS [avg_duration_(msec)], 
    CASE
	   WHEN execution_count = 0 THEN NULL
	   ELSE total_rows / execution_count
    END AS [avg_rows], 
    ph.query_plan, 
    qs.creation_time, 
    qs.last_execution_time, 
    CASE
	   WHEN ph.query_plan.exist ('declare namespace ns="http://schemas.microsoft.com/sqlserver/2004/07/showplan";(/ns:ShowPlanXML/ns:BatchSequence/ns:Batch/ns:Statements/ns:StmtCursor/ns:CursorPlan/@CursorRequestedType)[1]') = 0 THEN ''
	   ELSE ph.query_plan.value ('declare namespace ns="http://schemas.microsoft.com/sqlserver/2004/07/showplan";(/ns:ShowPlanXML/ns:BatchSequence/ns:Batch/ns:Statements/ns:StmtCursor/ns:CursorPlan/@CursorRequestedType)[1]', 'nvarchar (max)') 
    END AS cursor_type, 
    CASE
	   WHEN ph.query_plan.exist ('declare namespace ns="http://schemas.microsoft.com/sqlserver/2004/07/showplan";(/ns:ShowPlanXML/ns:BatchSequence/ns:Batch/ns:Statements/ns:StmtSimple/ns:QueryPlan/ns:MissingIndexes/ns:MissingIndexGroup)[1]') = 0 THEN ''
	   ELSE ph.query_plan.value ('declare namespace ns="http://schemas.microsoft.com/sqlserver/2004/07/showplan";(/ns:ShowPlanXML/ns:BatchSequence/ns:Batch/ns:Statements/ns:StmtSimple/ns:QueryPlan/ns:MissingIndexes/ns:MissingIndexGroup/@Impact)[1]', 'nvarchar (max)') 
    END AS missing_index_impact, 
    CASE
	   WHEN ph.query_plan.exist ('declare namespace ns="http://schemas.microsoft.com/sqlserver/2004/07/showplan";(/ns:ShowPlanXML/ns:BatchSequence/ns:Batch/ns:Statements/ns:StmtSimple/ns:QueryPlan/ns:MissingIndexes/ns:MissingIndexGroup/ns:MissingIndex/@Table)[1]') = 0 THEN ''
	   ELSE ph.query_plan.value ('declare namespace ns="http://schemas.microsoft.com/sqlserver/2004/07/showplan";(/ns:ShowPlanXML/ns:BatchSequence/ns:Batch/ns:Statements/ns:StmtSimple/ns:QueryPlan/ns:MissingIndexes/ns:MissingIndexGroup/ns:MissingIndex/@Table)[1]', 'nvarchar(max)') 
    END AS missing_index_table
  FROM sys.dm_exec_query_stats AS qs
       CROSS APPLY sys.dm_exec_sql_text (qs.sql_handle) AS st
                   CROSS APPLY sys.dm_exec_query_plan (qs.plan_handle) AS ph
  WHERE(st.text IS NOT NULL) 

    --AND (execution_count >= 100)  -- Count; change threshold
    --AND (total_logical_reads/execution_count >= 1000) -- Logical Reads; change threshold
    --AND (total_elapsed_time/execution_count) / 1000 >= 10 -- Duration (msec); change threshold
    AND DATEDIFF (dd, qs.last_execution_time, GETDATE ()) <= 7  -- Last Execution within period (days)
 
  ORDER BY total_worker_time DESC; 
GO

----------------------------------------------------------------------------------------------------

-- >> Missing Indexes <<

-- Filter-Values
declare @threshold_occurrence int, @threshold_impact int
set @threshold_occurrence = 0 --50 -- Avg. Proposals per Day online; change on demand
set @threshold_impact = 1 --90 -- Percent; change on demand

declare @daysonline int
select @daysonline = datediff(dd, create_date, getdate()) + 1 from sys.databases where name = 'tempdb'
declare @dbname sysname, @tsql nvarchar(max)
declare db_cur cursor fast_forward for
  select [name] from sys.databases 
  where [state] = 0 
    and [name] not in ('master', 'model', 'msdb', 'tempdb', 'SSI')  -- exclude databases
    and [name] in ('NAV_501') -- include databases 
  order by [name]
open db_cur
fetch next from db_cur into @dbname
while @@FETCH_STATUS = 0
begin
    set @tsql =
    '
    USE [' + @dbname + ']
    SELECT db_name() as [DB_Name]
    SELECT mig.index_handle, migs.user_seeks, migs.last_user_seek, migs.user_scans, migs.last_user_scan, migs.avg_user_impact,
		 CONVERT(decimal(18,2), migs.user_seeks * migs.avg_total_user_cost * (migs.avg_user_impact * 0.01)) AS [index_advantage],
		 db_name(mid.database_id) as db, object_name(mid.object_id) as object, mid.equality_columns, mid.inequality_columns, mid.included_columns,
		 ''CREATE INDEX [ssi_'' + convert(varchar, mig.index_handle) + ''_'' + replace(replace(replace(convert(varchar(30), getdate(), 120), ''-'', ''''), '':'', ''''), '' '', ''_'') + ''] ON ['' +  object_name(mid.object_id) + ''] '' + ''('' +
		  CASE WHEN mid.equality_columns is not null THEN mid.equality_columns ELSE '''' END +
		  CASE WHEN mid.equality_columns is null AND mid.inequality_columns is not null THEN mid.inequality_columns ELSE '''' END +
		  CASE WHEN mid.equality_columns is not null AND mid.inequality_columns is not null THEN '', '' + mid.inequality_columns ELSE '''' END + '')'' +
		  CASE WHEN mid.included_columns is not null THEN '' INCLUDE ('' + mid.included_columns + '')'' ELSE '''' END as tsql
    FROM sys.dm_db_missing_index_group_stats AS migs
    INNER JOIN sys.dm_db_missing_index_groups AS mig
	   ON (migs.group_handle = mig.index_group_handle)
    INNER JOIN sys.dm_db_missing_index_details AS mid
	   ON (mig.index_handle = mid.index_handle)
    WHERE (mid.database_id = db_id())
    AND ((migs.user_seeks / ' + convert(varchar(5), @daysonline) + ') >= ' + convert(varchar(5), @threshold_occurrence) + ')  
    AND (migs.avg_user_impact >= ' + convert(varchar(5), @threshold_impact) + ') 
    AND (datediff(dd, migs.last_user_seek, getdate()) <= 7) -- propsed within period 
    ORDER BY object_name(mid.object_id), mid.equality_columns, mid.inequality_columns, mid.included_columns
    '

    --print @tsql
    exec sp_executesql @tsql

    fetch next from db_cur into @dbname
end
close db_cur
deallocate db_cur
GO

----------------------------------------------------------------------------------------------------

  
