
-- PLAYGROUND --

-- Clear Prcedure-Cache
DBCC FREEPROCCACHE
GO

-- Record Count
SELECT COUNT(*) as "Total_Recs" FROM "dbo"."CRONUS AG$G_L Entry"
GO

-- Index Overview
EXEC sp_helpindex 'CRONUS AG$G_L Entry'
GO

-- Clustered-Index Details (Key: "Entry No.")
SELECT OBJECT_NAME([object_id]) AS [object], [index_id], [index_type_desc], [index_depth], [index_level], [page_count]  
FROM sys.dm_db_index_physical_stats ( 
    DB_ID()
   ,OBJECT_ID('CRONUS AG$G_L Entry')
   ,1  -- = $0, Primary Key, Clustered Index
   ,NULL
   ,'DETAILED'
)
GO

-- Non-Clustered-Index Details (Key: "G/L Account No.", "Posting Date")
SELECT OBJECT_NAME([object_id]) AS [object], [index_id], [index_type_desc], [index_depth], [index_level], [page_count]  
FROM sys.dm_db_index_physical_stats ( 
    DB_ID()
   ,OBJECT_ID('CRONUS AG$G_L Entry')
   ,2  -- = $1, Secondary Key, Non-Clustered Index
   ,NULL
   ,'DETAILED'
)
GO

-- Data Density, e.g. on "G/L Account No."
SELECT  "G_L Account No_", COUNT(*) as "Density" 
FROM "dbo"."CRONUS AG$G_L Entry" 
GROUP BY "G_L Account No_" 
ORDER BY 2 DESC
GO


----------------------------------------------------------------------------------------------------

-- EXECUTION BEHIND THE SCENES --

/*
    //NAV C/AL Code:

    //GLEntry.SETCURRENTKEY("G/L Account No.", "Posting Date");  // execute with and without SETCURRENTKEY
    GLEntry.SETRANGE("G/L Account No.", '1575');  // use different accounts with different density/selectivity (i.e. 1575, 4670, 4120)
    GLEntry.SETFILTER("Posting Date", '%1..%2', 010100D, 311210D);
    
    IF GLEntry.FINDSET THEN;
    IF GLEntry.FIND('-') THEN;
    IF GLEntry.FINDFIRST THEN;
    
    IF GLEntry.FINDLAST THEN;
    IF GLEntry.FIND('+') THEN;
    
    IF GLEntry.COUNT <> 0 THEN;
    IF GLEntry.ISEMPTY THEN;

*/

----------------------------------------------------------------------------------------------------

SET STATISTICS IO ON
--SET STATISTICS TIME ON
GO
-- To view Execution Plan: CTRL + M

-- Examples:
-- Account No.: 1575 (High Density   ==> Bad Selectivity)
-- Account No.: 4670 (Medium Density ==> Medium Selectivity)
-- Account No.: 3986 (Low Density    ==> Good Selectivity)

-- Applies to NAV 2013 and higher

/* FINDSET */
SELECT * FROM "dbo"."CRONUS AG$G_L Entry" WITH (READUNCOMMITTED)   
WHERE (("Posting Date">='2000-01-01 00:00:00' AND "Posting Date"<='2010-12-31 00:00:00')) AND (("G_L Account No_"= '3986'))
ORDER BY "Entry No_" OPTION (OPTIMIZE FOR UNKNOWN)
GO

/* FIND('-') */
SELECT TOP (50) * FROM "dbo"."CRONUS AG$G_L Entry" WITH (READUNCOMMITTED)   
WHERE (("Posting Date">='2000-01-01 00:00:00' AND "Posting Date"<='2010-12-31 00:00:00')) AND (("G_L Account No_"= '3986'))
ORDER BY "Entry No_" OPTION (OPTIMIZE FOR UNKNOWN)
GO
--SELECT * FROM "dbo"."CRONUS AG$G_L Entry" WITH (READUNCOMMITTED)   
--WHERE (("Posting Date">='2000-01-01 00:00:00' AND "Posting Date"<='2010-12-31 00:00:00')) AND (("G_L Account No_"= '3986'))
--AND "Entry No_" > ???
--ORDER BY "Entry No_" OPTION (OPTIMIZE FOR UNKNOWN)
--GO

/* FINDFIRST */
SELECT TOP 1 * FROM "dbo"."CRONUS AG$G_L Entry" WITH (READUNCOMMITTED)   
WHERE (("Posting Date">='2000-01-01 00:00:00' AND "Posting Date"<='2010-12-31 00:00:00')) AND (("G_L Account No_"= '3986'))
ORDER BY "Entry No_" OPTION (OPTIMIZE FOR UNKNOWN)
GO

/* FINDLAST */
SELECT TOP 1 * FROM "dbo"."CRONUS AG$G_L Entry" WITH (READUNCOMMITTED)   
WHERE (("Posting Date">='2000-01-01 00:00:00' AND "Posting Date"<='2010-12-31 00:00:00')) AND (("G_L Account No_"= '3986'))
ORDER BY "Entry No_" DESC OPTION (OPTIMIZE FOR UNKNOWN)
GO

/* FIND('+') */
SELECT TOP (50) * FROM "dbo"."CRONUS AG$G_L Entry" WITH (READUNCOMMITTED)   
WHERE (("Posting Date">='2000-01-01 00:00:00' AND "Posting Date"<='2010-12-31 00:00:00')) AND (("G_L Account No_"= '3986'))
ORDER BY "Entry No_" DESC OPTION (OPTIMIZE FOR UNKNOWN)
GO

/* COUNT */
SELECT COUNT(*) FROM "dbo"."CRONUS AG$G_L Entry" WITH (READUNCOMMITTED)   
WHERE (("Posting Date">='2000-01-01 00:00:00' AND "Posting Date"<='2010-12-31 00:00:00')) AND (("G_L Account No_"= '3986'))
GO

/* ISEMPTY */
SELECT TOP 1 NULL FROM "dbo"."CRONUS AG$G_L Entry" WITH (READUNCOMMITTED)   
WHERE (("Posting Date">='2000-01-01 00:00:00' AND "Posting Date"<='2010-12-31 00:00:00')) AND (("G_L Account No_"= '3986'))
GO



